import { Provider, Type } from '@angular/core';
import { ControlValueAccessor } from '@angular/forms';
import { noop } from '@s-libs/micro-dash';
import { DirectiveSuperclass } from './directive-superclass';
/**
 * Use in the `providers` of a component that implements `ControlValueAccessor` to reduce some boilerplate.
 *
 * ```ts
 * @Component({ providers: [provideValueAccessor(MyFormControl)] }
 * class MyFormControl extends BaseFormControl {
 *   // ...
 * }
 * ```
 */
export declare function provideValueAccessor(type: Type<any>): Provider;
/**
 * Extend this when creating a form control to reduce some boilerplate.
 *
 * This example allows 2-way binding to a number via `[(ngModel)]`, `[formControl]`, or any other technique that leverages the `ControlValueAccessor` interface.
 * ```ts
 * @Component({
 *   template: `
 *     <button (click)="increment()" [disabled]="isDisabled">{{ counter }}</button>
 *   `,
 *   providers: [provideValueAccessor(CounterComponent)],
 * })
 * class CounterComponent extends FormComponentSuperclass<number> {
 *   counter = 0;
 *
 *   handleIncomingValue(value: number) {
 *     this.counter = value;
 *   }
 *
 *   increment() {
 *     this.emitOutgoingValue(++this.counter);
 *     this.onTouched();
 *   }
 * }
 * ```
 */
export declare abstract class FormComponentSuperclass<T> extends DirectiveSuperclass implements ControlValueAccessor {
    /** Call this to emit a new value when it changes. */
    emitOutgoingValue: (value: T) => void;
    /** Call this to "commit" a change, traditionally done e.g. on blur. */
    onTouched: typeof noop;
    /** You can bind to this in your template as needed. */
    isDisabled: boolean;
    /** Implement this to handle a new value coming in from outside. */
    abstract handleIncomingValue(value: T): void;
    /** Called as angular propagates value changes to this `ControlValueAccessor`. You normally do not need to use it. */
    writeValue(value: T): void;
    /** Called as angular sets up the binding to this `ControlValueAccessor`. You normally do not need to use it. */
    registerOnChange(fn: (value: T) => void): void;
    /** Called as angular sets up the binding to this `ControlValueAccessor`. You normally do not need to use it. */
    registerOnTouched(fn: VoidFunction): void;
    /** Called as angular propagates disabled changes to this `ControlValueAccessor`. You normally do not need to use it. */
    setDisabledState(isDisabled: boolean): void;
}
