import { Injector, OnInit } from '@angular/core';
import { AbstractControl } from '@angular/forms';
import { Observable } from 'rxjs';
import { FormComponentSuperclass } from './form-component-superclass';
import * as i0 from "@angular/core";
/**
 * Extend this when creating a form component that simply wraps existing ones, to reduce a lot of boilerplate. **Warning:** You _must_ include a constructor in your subclass.
 *
 * To wrap a single form control use the subclass {@linkcode WrappedFormControlSuperclass}:
 * ```ts
 * @Component({
 *   template: `<input [formControl]="control">`,
 *   providers: [provideValueAccessor(StringComponent)],
 * })
 * class StringComponent extends WrappedFormControlSuperclass<string> {}
 * ```
 *
 * Example of wrapping multiple inner components:
 * ```ts
 * class FullName {
 *   firstName = '';
 *   lastName = '';
 * }
 *
 * @Component({
 *   selector: 'app-full-name',
 *   template: `
 *     <div [formGroup]="control">
 *       <input id="first" formControlName="firstName" />
 *       <input id="last" formControlName="lastName" />
 *     </div>
 *   `,
 *   providers: [provideValueAccessor(FullNameComponent)],
 * })
 * class FullNameComponent extends WrappedControlSuperclass<FullName> {
 *   control = new FormGroup({
 *     firstName: new FormControl(),
 *     lastName: new FormControl(),
 *   });
 *
 *   protected outerToInner(outer: FullName | null): FullName {
 *     // `outer` can come in as `null` during initialization when the user binds with `ngModel`
 *     return outer || new FullName();
 *   }
 * }
 * ```
 *
 * Example when you need to modify the wrapped control's value:
 * ```ts
 * @Component({
 *   template: `<input type="datetime-local" [control]="formControl">`,
 *   providers: [provideValueAccessor(DateComponent)],
 * })
 * class DateComponent extends WrappedFormControlSuperclass<Date, string> {
 *   protected innerToOuter(inner: string): Date {
 *     return new Date(inner + "Z");
 *   }
 *
 *   protected outerToInner(outer: Date): string {
 *     if (outer === null) {
 *       return ""; // happens during initialization
 *     }
 *     return outer.toISOString().substr(0, 16);
 *   }
 * }
 * ```
 */
export declare abstract class WrappedControlSuperclass<OuterType, InnerType = OuterType> extends FormComponentSuperclass<OuterType> implements OnInit {
    #private;
    /** Bind this to your inner form control to make all the magic happen. */
    abstract control: AbstractControl;
    constructor(injector: Injector);
    ngOnInit(): void;
    /** Called as angular propagates values changes to this `ControlValueAccessor`. You normally do not need to use it. */
    handleIncomingValue(outer: OuterType): void;
    /** Called as angular propagates disabled changes to this `ControlValueAccessor`. You normally do not need to use it. */
    setDisabledState(isDisabled: boolean): void;
    /**
     * Override this to modify a value coming from the outside to the format needed within this component.
     *
     * For more complex needs, see {@linkcode #setUpOuterToInner$} instead.
     */
    protected outerToInner(outer: OuterType): InnerType;
    /**
     * Override this to for full control over the stream of values passed in to your subclass.
     *
     * In this example, incoming values are debounced before being passed through to the inner form control
     * ```ts
     * setUpOuterToInner$(outer$: Observable<OuterType>): Observable<InnerType> {
     *   return outer$.pipe(
     *     debounce(300),
     *     map((outer) => doExpensiveTransformToInnerValue(outer)),
     *   );
     * }
     * ```
     *
     * For a simple transformation, see {@linkcode #outerToInner} instead.
     */
    protected setUpOuterToInner$(outer$: Observable<OuterType>): Observable<InnerType>;
    /**
     * Override this to modify a value coming from within this component to the format expected on the outside.
     *
     * For more complex needs, see {@linkcode #setUpInnerToOuter$} instead.
     */
    protected innerToOuter(inner: InnerType): OuterType;
    /**
     * Override this to for full control over the stream of values emitted from your subclass.
     *
     * In this example, illegal values are not emitted
     * ```ts
     * setUpInnerToOuter$(inner$: Observable<InnerType>): Observable<OuterType> {
     *   return inner$.pipe(
     *     debounce(300),
     *     filter((inner) => isLegalValue(outer)),
     *   );
     * }
     * ```
     *
     * For a simple transformation, see {@linkcode #innerToOuter} instead.
     */
    protected setUpInnerToOuter$(inner$: Observable<InnerType>): Observable<OuterType>;
    static ɵfac: i0.ɵɵFactoryDeclaration<WrappedControlSuperclass<any, any>, never>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<WrappedControlSuperclass<any, any>, never, never, {}, {}, never>;
}
